{% set timeout = 420 %}

- test:
    timeout:
      minutes: {{ timeout }}
    docker:
      image: git.morello-project.org:5050/morello/morello-ci-containers/morello-android-platform-tools
      network_from: "{{ tmpdir.name }}"
    definitions:
    - repository: https://git.morello-project.org/morello/morello-ci-pipelines.git
      from: git
      path: lava/test-definitions/logd/testdef.yaml
      name: logd
      parameters:
        USERDATA_URL: "{{ parameters.USERDATA }}"
